/*
	
	StatWatchWindow.cpp
	
	Copyright 1997 CHS Systems (CHeSSy) and Chris Heimark, All Rights Reserved.

	http://www.webleap.com
	
	be-technoid@webleap.com	
*/

#ifndef _APPLICATION_H
#include <Application.h>
#endif

#ifndef STAT_WATCH_WINDOW_H
#include "StatWatchWindow.h"
#endif

#include <stdio.h>
#include <memory.h>

// screen position constant rects
const BRect kKernalNameFrame0 (5, 4,  295, 14);
const BRect kKernalNameFrame1 (5, 20, 295, 30);

const BRect kThreadsCurrentStatusFrame (5, 30, 295, 55);
const BRect kThreadsPeakStatusFrame (5, 60, 295, 85);
const BRect kTeamsCurrentStatusFrame (5, 100, 295, 125);
const BRect kTeamsPeakStatusFrame (5, 130, 295, 155);
const BRect kSemsCurrentStatusFrame (5, 170, 295, 195);
const BRect kSemsPeakStatusFrame (5, 200, 295, 225);
const BRect kPagesCurrentStatusFrame (5, 240, 295, 265);
const BRect kPagesPeakStatusFrame (5, 270, 295, 295);
const BRect kPortsCurrentStatusFrame (5, 310, 295, 335);
const BRect kPortsPeakStatusFrame (5, 340, 295, 365);

void set_palette_entry(long i,rgb_color c);

StatWatchWindow::StatWatchWindow (BRect frame) : BWindow (frame, 
															"StatWatch", 
															B_TITLED_WINDOW, 
															B_NOT_V_RESIZABLE | B_NOT_ZOOMABLE)
{
	char			temp  [B_FILE_NAME_LENGTH];
	rgb_color 		rgb;

	BStringView		*basicInfo;

	system_info		s;
	
	// get some basic stuff that changes only once
	status_t 		status = get_system_info (&s);


	// compute and just display it once
	sprintf (temp, "Platform: %s/%s  Build: %s %s", 
					s.platform_type == B_BEBOX_PLATFORM ? "BeBox" : "Mac",
					s.kernel_name, 
					s.kernel_build_date, 
					s.kernel_build_time
											);
	// add view to window
	basicInfo = new BStringView ( kKernalNameFrame0, NULL, temp, B_FULL_UPDATE_ON_RESIZE | B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP);
	basicInfo->SetAlignment (B_ALIGN_CENTER);
	AddChild (basicInfo);

	// compute and just display it once
	sprintf (temp, "%ldMHz %s CPU[x%ld] / %ldMHz Bus", 
					(long) (s.cpu_clock_speed / 1000000),
					GetAsciiCPU (s.cpu_type),
					s.cpu_count,
					(long) (s.bus_clock_speed / 1000000)
													);
	// add view to window
	basicInfo = new BStringView (kKernalNameFrame1, NULL, temp, B_FULL_UPDATE_ON_RESIZE | B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP);
	basicInfo->SetAlignment (B_ALIGN_CENTER);
	AddChild (basicInfo);

	// setup green status bars
	rgb.red = 0;	rgb.green = 255;	rgb.blue = 0;	rgb.alpha = 0;


	// current threads
	mCurrentThreads = new BStatusBar (kThreadsCurrentStatusFrame, "CurrentThreads");
	mCurrentThreads->SetResizingMode (B_FULL_UPDATE_ON_RESIZE | B_FOLLOW_LEFT_RIGHT);
	AddChild (mCurrentThreads);
	mCurrentThreads->SetBarColor (rgb);
	// maximum threads
	mMaximumThreads = new BStatusBar (kThreadsPeakStatusFrame, "MaxThreads");
	mMaximumThreads->SetResizingMode (B_FULL_UPDATE_ON_RESIZE | B_FOLLOW_LEFT_RIGHT);
	AddChild (mMaximumThreads);

	// current teams
	mCurrentTeams = new BStatusBar (kTeamsCurrentStatusFrame, "CurrentTeams");
	mCurrentTeams->SetResizingMode (B_FULL_UPDATE_ON_RESIZE | B_FOLLOW_LEFT_RIGHT);
	AddChild (mCurrentTeams);
	mCurrentTeams->SetBarColor (rgb);
	// maximum teams
	mMaximumTeams = new BStatusBar (kTeamsPeakStatusFrame, "MaxTeams");
	mMaximumTeams->SetResizingMode (B_FULL_UPDATE_ON_RESIZE | B_FOLLOW_LEFT_RIGHT);
	AddChild (mMaximumTeams);

	// current semaphores
	mCurrentSems = new BStatusBar (kSemsCurrentStatusFrame, "CurrentSems");
	mCurrentSems->SetResizingMode (B_FULL_UPDATE_ON_RESIZE | B_FOLLOW_LEFT_RIGHT);
	AddChild (mCurrentSems);
	mCurrentSems->SetBarColor (rgb);
	// maximum semaphores
	mMaximumSems = new BStatusBar (kSemsPeakStatusFrame, "MaxSems");
	mMaximumSems->SetResizingMode (B_FULL_UPDATE_ON_RESIZE | B_FOLLOW_LEFT_RIGHT);
	AddChild (mMaximumSems);

	// current pages
	mCurrentPages = new BStatusBar (kPagesCurrentStatusFrame, "CurrentPages");
	mCurrentPages->SetResizingMode (B_FULL_UPDATE_ON_RESIZE | B_FOLLOW_LEFT_RIGHT);
	AddChild (mCurrentPages);
	mCurrentPages->SetBarColor (rgb);
	// maximum pages
	mMaximumPages = new BStatusBar (kPagesPeakStatusFrame, "MaxPages");
	mMaximumPages->SetResizingMode (B_FULL_UPDATE_ON_RESIZE | B_FOLLOW_LEFT_RIGHT);
	AddChild (mMaximumPages);

	// current ports
	mCurrentPorts = new BStatusBar (kPortsCurrentStatusFrame, "CurrentPorts");
	mCurrentPorts->SetResizingMode (B_FULL_UPDATE_ON_RESIZE | B_FOLLOW_LEFT_RIGHT);
	AddChild (mCurrentPorts);
	mCurrentPorts->SetBarColor (rgb);
	// maximum ports
	mMaximumPorts = new BStatusBar (kPortsPeakStatusFrame, "MaxPorts");
	mMaximumPorts->SetResizingMode (B_FULL_UPDATE_ON_RESIZE | B_FOLLOW_LEFT_RIGHT);
	AddChild (mMaximumPorts);

	// zap our tracking information
	mMaxThreadsUsed = mMaxTeamsUsed = mMaxSemsUsed = mMaxPagesUsed = mMaxPortsUsed = 0;
	mPriorThreadsUsed = mPriorTeamsUsed = mPriorSemsUsed = mPriorPagesUsed = mPriorPortsUsed = 0;

	// don't wait for our first update
	DisplayOurStats ();
	
	// make window visible
	Show();
}

bool StatWatchWindow::QuitRequested()
{
	be_app->PostMessage (B_QUIT_REQUESTED);
	return true;
}

void StatWatchWindow::MessageReceived (BMessage* message)
{
	switch (message->what)
	{
		case	SW_UPDATE_STATS:
			{
				// display our statistics
				DisplayOurStats ();
			}
			break;

		default:

			BWindow::MessageReceived (message);
			break;
	}
}

void StatWatchWindow::DisplayOurStats ()
{
	char		temp  [B_FILE_NAME_LENGTH];
	char		temp1 [B_FILE_NAME_LENGTH];

	system_info		s;
	
	// get system statistics right now
	status_t 		status = get_system_info (&s);

	// threads
	if (s.used_threads > mMaxThreadsUsed)
	{
		mMaxThreadsUsed = s.used_threads;
	}
	if (s.used_threads != mPriorThreadsUsed)
	{
		mPriorThreadsUsed = s.used_threads;

		mCurrentThreads->SetMaxValue (mMaxThreadsUsed);
		sprintf (temp1, "THREADS: %ld", s.used_threads);
		sprintf (temp, "Peak: %ld", (long) mMaxThreadsUsed);
		mCurrentThreads->Update ((float) s.used_threads - mCurrentThreads->CurrentValue (), temp1, temp);
	
		mMaximumThreads->SetMaxValue ((float) s.max_threads);
		sprintf (temp1, "Peak: %ld", (long) mMaxThreadsUsed);
		sprintf (temp, "Max: %ld", s.max_threads);
		mMaximumThreads->Update (mMaxThreadsUsed - mMaximumThreads->CurrentValue (), temp1, temp);

		SetBarColor (mMaximumThreads, mMaxThreadsUsed, s.max_threads);
	}
	// teams
	if (s.used_teams > mMaxTeamsUsed)
	{
		mMaxTeamsUsed = s.used_teams;
	}
	if (s.used_teams != mPriorTeamsUsed)
	{
		mPriorTeamsUsed = s.used_teams;

		mCurrentTeams->SetMaxValue (mMaxTeamsUsed);
		sprintf (temp1, "TEAMS: %ld", s.used_teams);
		sprintf (temp, "Peak: %ld", (long) mMaxTeamsUsed);
		mCurrentTeams->Update ((float) s.used_teams - mCurrentTeams->CurrentValue (), temp1, temp);
	
		mMaximumTeams->SetMaxValue ((float) s.max_teams);
		sprintf (temp1, "Peak: %ld", (long) mMaxTeamsUsed);
		sprintf (temp, "Max: %ld", s.max_teams);
		mMaximumTeams->Update (mMaxTeamsUsed - mMaximumTeams->CurrentValue (), temp1, temp);

		SetBarColor (mMaximumTeams, mMaxTeamsUsed, s.max_teams);
	}
	// semaphores
	if (s.used_sems > mMaxSemsUsed)
	{
		mMaxSemsUsed = s.used_sems;
	}
	if (s.used_sems != mPriorSemsUsed)
	{
		mPriorSemsUsed = s.used_sems;

		mCurrentSems->SetMaxValue (mMaxSemsUsed);
		sprintf (temp1, "SEMAPHORES: %ld", s.used_sems);
		sprintf (temp, "Peak: %ld", (long) mMaxSemsUsed);
		mCurrentSems->Update ((float) s.used_sems - mCurrentSems->CurrentValue (), temp1, temp);
	
		mMaximumSems->SetMaxValue ((float) s.max_sems);
		sprintf (temp1, "Peak: %ld", (long) mMaxSemsUsed);
		sprintf (temp, "Max: %ld", s.max_sems);
		mMaximumSems->Update (mMaxSemsUsed - mMaximumSems->CurrentValue (), temp1, temp);

		SetBarColor (mMaximumSems, mMaxSemsUsed, s.max_sems);
	}
	// pages
	if (s.used_pages > mMaxPagesUsed)
	{
		mMaxPagesUsed = s.used_pages;
	}
	if (s.used_pages != mPriorPagesUsed)
	{
		mPriorPagesUsed = s.used_pages;

		mCurrentPages->SetMaxValue (mMaxPagesUsed);
		sprintf (temp1, "PAGES: %ld (%ld MB) / Faults: %ld", s.used_pages, ((B_PAGE_SIZE / 1024) * s.used_pages) / 1024, s.page_faults);
		sprintf (temp, "Peak: %ld", (long) mMaxPagesUsed);
		mCurrentPages->Update ((float) s.used_pages - mCurrentPages->CurrentValue (), temp1, temp);
	
		mMaximumPages->SetMaxValue ((float) s.max_pages);
		sprintf (temp1, "Peak: %ld", (long) mMaxPagesUsed);
		sprintf (temp, "Max: %ld (%ld MB)", s.max_pages, ((B_PAGE_SIZE / 1024) * s.max_pages) / 1024);
		mMaximumPages->Update (mMaxPagesUsed - mMaximumPages->CurrentValue (), temp1, temp);

		SetBarColor (mMaximumPages, mMaxPagesUsed, s.max_pages);
	}
	// ports
	if (s.used_ports > mMaxPortsUsed)
	{
		mMaxPortsUsed = s.used_ports;
	}
	if (s.used_ports != mPriorPortsUsed)
	{
		mPriorPortsUsed = s.used_ports;

		mCurrentPorts->SetMaxValue (mMaxPortsUsed);
		sprintf (temp1, "PORTS: %ld", s.used_ports);
		sprintf (temp, "Peak: %ld", (long) mMaxPortsUsed);
		mCurrentPorts->Update ((float) s.used_ports - mCurrentPorts->CurrentValue (), temp1, temp);

		mMaximumPorts->SetMaxValue ((float) s.max_ports);
		sprintf (temp1, "Peak: %ld", (long) mMaxPortsUsed);
		sprintf (temp, "Max: %ld", s.max_ports);
		mMaximumPorts->Update (mMaxPortsUsed - mMaximumPorts->CurrentValue (), temp1, temp);

		SetBarColor (mMaximumPorts, mMaxPortsUsed, s.max_ports);
	}
}

char *StatWatchWindow::GetAsciiCPU (cpu_type cpuType) const
{
	// array of CPU names / note array usage below!
	static char	*cpus[] = { "601", "603", "603e", "604", "604e", "Other" };

	int		cpu;

	switch (cpuType)
	{
		case	B_CPU_PPC_601:		cpu = 0;					break;
	
		case	B_CPU_PPC_603:		cpu = 1;					break;
	
		case	B_CPU_PPC_603e:		cpu = 2;					break;
	
		case	B_CPU_PPC_604:		cpu = 3;					break;
	
		case	B_CPU_PPC_604e:		cpu = 4;					break;
		
		default:					cpu = sizeof (cpus) - 1;	break;
	}
	
	return cpus[cpu];
}

void StatWatchWindow::SetBarColor (BStatusBar *bar, float value, float max)
{
	short		percent;
	rgb_color 	rgb;

	// setup for any color
	memset (&rgb, 0, sizeof (rgb));

	// figure percentage
	percent = (short) (value / max * 100.0);
	
	// green - OK
	if (percent <= 33)
	{
		rgb.green = 255;
	}
	// yellow - CAUTION
	else if (percent <= 75)
	{
		rgb.red = 255;
		rgb.green  = 255;
	}
	// red - UH OH - BUSY BOX
	else
	{
		rgb.red = 255;
	}
	// make it so
	bar->SetBarColor (rgb);
}
